@@ -5,7 +5,7 @@
 
 #define CRYPT
 
-int	Null_Key();
+int	Null_Key(void);
 int     Set_Key(char* key);
 u_char* Encrypt(u_char* in, int* len);
 u_char* Encrypt_Ctrl(u_char* in, int* len);
