@@ -1,5 +1,5 @@
 #
-# Copyright (c) 1995,1996,1997 University College London
+# Copyright (c) 1995-98 University College London
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -33,17 +33,17 @@
 # SUCH DAMAGE.
 #
 
-option add *background 			gray80 		widgetDefault
-option add *foreground 			black 		widgetDefault
-option add *activeBackground 		gray85 		widgetDefault
-option add *selectBackground 		gray85 		widgetDefault
-option add *Scale.sliderForeground 	gray80 		widgetDefault
-option add *Scale.activeForeground 	gray85 		widgetDefault
-option add *Scale.background 		gray80		widgetDefault
-option add *Entry.background 		gray70 		widgetDefault
-option add *Menu*selectColor 		forestgreen 	widgetDefault
-option add *Radiobutton*selectColor 	forestgreen 	widgetDefault
-option add *Checkbutton*selectColor 	forestgreen 	widgetDefault
+option add *background 			gray80 		
+option add *foreground 			black 	
+option add *activeBackground 		gray85 
+option add *selectBackground 		gray85
+option add *Scale.sliderForeground 	gray80 	
+option add *Scale.activeForeground 	gray85 		
+option add *Scale.background 		gray80
+option add *Entry.background 		gray70
+option add *Menu*selectColor 		forestgreen
+option add *Radiobutton*selectColor 	forestgreen
+option add *Checkbutton*selectColor 	forestgreen 
 option add *borderWidth 		1 
 
 set statsfont     -*-courier-medium-r-*-*-12-*-*-*-*-*-iso8859-1
@@ -52,13 +52,18 @@ set infofont      -*-helvetica-medium-r-normal--12-*-p-*-iso8859-1
 set smallfont     -*-helvetica-medium-r-normal--10-*-p-*-iso8859-1
 set verysmallfont -*-courier-medium-o-normal--8-*-m-*-iso8859-1
 
+# app is common rather than rat so info is shared between all win32 mbone apps.
 set V(class) "Mbone Applications"
-set V(app)   "rat"
+set V(app)   "common"
 
-if { [string compare [info procs putregistry] "putregistry"] == 0} {
-    set win32 1
+if { [string compare [info commands registry] "registry"] == 0 } {
+	set win32 1
+	if { [array names env HOMEDIR] != "HOMEDIR" } {
+# Fix expansion of ~ if HOMEDIR does not exist.
+		set env(HOMEDIR) .
+	}
 } else {
-    set win32 0
+	set win32 0
 }
 
 #dropdown list
@@ -235,13 +240,27 @@ pack .l.t.list -side left -fill both -expand 1
 button .r.c.vol.t1 -highlightthickness 0 -padx 0 -pady 0 -text mute -command toggle_play 
 button .r.c.vol.l1 -highlightthickness 0 -padx 0 -pady 0 -command toggle_output_port -bitmap "speaker"
 bargraph .r.c.vol.b1
-scale .r.c.vol.s1 -highlightthickness 0 -font $verysmallfont -from 99 -to 0 -command set_vol -orient vertical -relief raised 
+scale .r.c.vol.s1 -highlightthickness 0 -font $verysmallfont -from 99 -to 0 -command tcl_set_vol -orient vertical -relief raised 
 
 pack .r.c.vol.t1 -side top -fill x
 pack .r.c.vol.l1 -side top -fill x
 pack .r.c.vol.b1 -side left -fill y
 pack .r.c.vol.s1 -side right -fill y
 
+proc tcl_set_vol {vol} {
+	set_vol $vol
+	if { [.r.c.vol.s1 get] != $vol } {
+		.r.c.vol.s1 set $vol
+	}
+}
+
+proc tcl_set_gain {gain} {
+	set_gain $gain
+	if { [.r.c.gain.s2 get] != $gain } {
+		.r.c.gain.s2 set $gain
+	}
+}
+
 # Device input controls
 button .r.c.gain.t2 -highlightthickness 0 -padx 0 -pady 0 -text mute -command toggle_send -relief sunken
 button .r.c.gain.l2 -highlightthickness 0 -padx 0 -pady 0 -command toggle_input_port -bitmap "mic_mute"
@@ -599,7 +618,7 @@ wm withdraw .copyright
 message   .copyright.m -text {
  Robust-Audio Tool (RAT)
  
- Copyright (C) 1995,1996,1997 University College London
+ Copyright (C) 1995-98 University College London
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
@@ -659,13 +678,16 @@ pack   .copyright.m .copyright.d -side top -fill x -expand 1
 wm title     .copyright "RAT Copyright"
 wm resizable .copyright 0 0
 
-if {[glob ~] == "/"} {
-	set rtpfname /.RTPdefaults
+if { 0 == $win32 } {
+    if {[glob ~] == "/"} {
+		set rtpfname /.RTPdefaults
+    } else {
+		set rtpfname ~/.RTPdefaults
+    }
 } else {
-	set rtpfname ~/.RTPdefaults
+    set rtpfname .RTPdefaults
 }
 
-
 proc savename {} {
     global rtpfname rtcp_name rtcp_email rtcp_phone rtcp_loc V win32
     if {$win32 == 0 } {
@@ -676,10 +698,10 @@ proc savename {} {
 	if {$rtcp_loc   != ""} {puts $f "*rtpLoc:   $rtcp_loc"}
 	close $f
     } else {
-	putregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpName"  "$rtcp_name"
-	putregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpEmail"  "$rtcp_email"
-	putregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpPhone"  "$rtcp_phone"
-	putregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpLoc"  "$rtcp_loc"
+	registry set "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpName"   "$rtcp_name"
+	registry set "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpEmail"  "$rtcp_email"
+	registry set "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpPhone"  "$rtcp_phone"
+	registry set "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpLoc"    "$rtcp_loc"
     }
 }
 
@@ -702,10 +724,10 @@ if {$win32 == 0} {
     }
 } else {
     if {$rtcp_name == ""} {
-	catch {set rtcp_name  [getregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpName"]  } 
-	catch {set rtcp_email [getregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpEmail"] } 
-	catch {set rtcp_phone [getregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpPhone"] } 
-	catch {set rtcp_loc   [getregistry "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpLoc"  ] } 
+	catch {set rtcp_name  [registry get "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpName"]  } 
+	catch {set rtcp_email [registry get "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpEmail"] } 
+	catch {set rtcp_phone [registry get "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpPhone"] } 
+	catch {set rtcp_loc   [registry get "HKEY_CURRENT_USER\\Software\\$V(class)\\$V(app)" "*rtpLoc"  ] } 
     }
 }
 
@@ -739,8 +761,6 @@ if {$rtcp_name == ""} {
 
 update
 
-catch {
-    source ~/.rat
-}
 
+after 500 "catch {source ~/.rat}"
 
