@@ -4,8 +4,8 @@
  * Modified: Vicky Hardman + Colin Perkins
  * Purpose:  RTCP protocol routines
  *
- * $Revision: 1.19 $
- * $Date: 1997/05/23 12:52:00 $
+ * $Revision: 1.2 $
+ * $Date: 1998/08/18 14:17:44 $
  *
  * Copyright (c) 1995,1996,1997 University College London
  * All rights reserved.
@@ -87,10 +87,12 @@ rtcp_check_rtcp_pkt(u_int8 *packet, int len)
         /*   the overall length of the compound RTCP packet as received.    */
         /*   This is a fairly strong check.                                 */
 
-	rtcp_t	*pkt = (rtcp_t *) packet;
-	rtcp_t	*end = (rtcp_t *) (((char *) pkt) + len);
-	rtcp_t	*r   = pkt;
-	int	 l   = 0;
+	rtcp_t	*pkt  = (rtcp_t *) packet;
+	rtcp_t	*end  = (rtcp_t *) (((char *) pkt) + len);
+	rtcp_t	*r    = pkt;
+	int	 l    = 0;
+	int	 last = 0;
+
 
 	/* All RTCP packets must be compound packets (RFC1889, section 6.1) */
 	if (((ntohs(pkt->common.length) + 1) * 4) == len) return FALSE;
@@ -106,7 +108,10 @@ rtcp_check_rtcp_pkt(u_int8 *packet, int len)
 	/* the last packet.                                                      */
 	do {
 		if (r->common.type != 2) return FALSE;
-		if ((r->common.p != 0) && ((r + 1) != end)) return FALSE;
+		if (last == 1) {
+			return FALSE;
+		}
+		if (r->common.p == 1) last = 1;
 		l += (ntohs(r->common.length) + 1) * 4;
 		r  = (rtcp_t *) (((u_int32 *) r) + ntohs(r->common.length) + 1);
 	} while (r < end);
@@ -143,6 +148,7 @@ rtcp_decode_rtcp_pkt(session_struct *sp, session_struct *sp2, u_int8 *packet, in
 		switch (pkt->common.pt) {
 		case RTCP_SR:
 			ssrc = ntohl(pkt->r.sr.ssrc);
+			if ((ssrc == sp->db.myssrc) && sp->filter_loopback) return;
 			dbe = rtcp_getornew_dbentry(sp, ssrc, addr, cur_time);
 			dbe->last_active   = sp->cur_time;
 			dbe->last_sr       = sp->cur_time;
@@ -155,7 +161,9 @@ rtcp_decode_rtcp_pkt(session_struct *sp, session_struct *sp2, u_int8 *packet, in
 
 			break;
 		case RTCP_RR:	/* We won't deal with this just yet */
-			dbe = rtcp_getornew_dbentry(sp, ntohl(pkt->r.rr.ssrc), addr, cur_time);
+			ssrc = ntohl(pkt->r.rr.ssrc);
+			if (ssrc == sp->db.myssrc) return;
+			dbe = rtcp_getornew_dbentry(sp, ssrc, addr, cur_time);
 			dbe->last_active = sp->cur_time;
 			break;
 		case RTCP_BYE:
@@ -351,7 +359,9 @@ rtcp_packet_fmt_sdes(session_struct *sp, u_int8 * ptr)
 			         abort();
 			}
 		} else {
-			len += rtcp_add_sdes_item(&ptr[len], RTCP_SDES_NAME, sp->db.sdes[RTCP_SDES_NAME-1]);
+			if (sp->db.sdes[RTCP_SDES_NAME-1] != NULL) {
+				len += rtcp_add_sdes_item(&ptr[len], RTCP_SDES_NAME, sp->db.sdes[RTCP_SDES_NAME-1]);
+			}
 		}
 	}
 
@@ -668,8 +678,8 @@ rtcp_forward(rtcp_t *pckt, session_struct *sp1, session_struct *sp2)
 	}
 
 	ptr = rtcp_packet_fmt_srrr(sp2, ptr);
-	memcpy(ptr, pckt, (pckt->common.length+1)*4);
-	ptr += (pckt->common.length+1)*4;
+	memcpy(ptr, pckt, (ntohs(pckt->common.length)+1)*4);
+	ptr += (ntohs(pckt->common.length)+1)*4;
 	packlen = ptr - (u_int8 *) packet;
 	net_write(sp1->rtcp_fd, sp1->net_maddress, sp1->rtcp_port, (u_int8 *) packet, packlen, PACKET_RTCP);
 	sp2->db.last_rpt = now;
