@@ -3,8 +3,8 @@
  *	PROGRAM: Rat
  *	AUTHOR: Isidor Kouvelas + Colin Perkins + Orion Hodson
  * 	
- *	$Revision: 1.52 $
- * 	$Date: 1997/05/22 10:57:32 $
+ *	$Revision: 1.2 $
+ * 	$Date: 1998/09/22 08:56:20 $
  *
  * Copyright (c) 1995,1996,1997 University College London
  * All rights reserved.
@@ -47,10 +47,11 @@
 #include "crypt.h"
 
 #ifdef WIN32
-extern "C" int WinPutRegistry(ClientData, Tcl_Interp*, int ac, char** av);
-extern "C" int WinGetRegistry(ClientData, Tcl_Interp*, int ac, char** av);
-extern "C" int WinPutsCmd(ClientData, Tcl_Interp*, int ac, char** av);
-extern "C" int WinGetUserName(ClientData, Tcl_Interp*, int ac, char** av);
+extern int WinPutRegistry(ClientData, Tcl_Interp*, int ac, char** av);
+extern int WinGetRegistry(ClientData, Tcl_Interp*, int ac, char** av);
+extern int WinPutsCmd(ClientData, Tcl_Interp*, int ac, char** av);
+extern int WinGetUserName(ClientData, Tcl_Interp*, int ac, char** av);
+extern int Registry_Init(Tcl_Interp*);
 #endif
 
 Tcl_Interp *interp;	/* Interpreter for application. */
@@ -852,10 +853,7 @@ ui_update(session_struct *session_pointer)
 		if (audio_duplex(session_pointer->audio_fd) == FALSE) {
 			session_pointer->voice_switching = MIKE_MUTES_NET;
 			eval_check(interp, simplex_script);
-#ifdef DEBUG
-		} else {
-			printf("Full duplex audio!\n");
-#endif
+
 		}
 	}
 
@@ -1124,11 +1122,10 @@ ui_init(session_struct *session_pointer, int argc, char **argv)
 	Tcl_CreateCommand(interp, "mute",		mute_cmd, 		(ClientData) session_pointer, NULL);
 	Tcl_CreateCommand(interp, "get_audio",          get_audio_cmd,          (ClientData) session_pointer, NULL);
 #ifdef WIN32
-	tcl.CreateCommand("putregistry", WinPutRegistry, (ClientData)session_pointer, NULL);
-        tcl.CreateCommand("getregistry", WinGetRegistry, (ClientData)session_pointer, NULL);
-	tcl.CreateCommand("puts", WinPutsCmd, (ClientData)session_pointer, NULL);
-        tcl.CreateCommand("getusername", WinGetUserName, (ClientData)session_pointer, NULL);
-#endif
+	Registry_Init(interp);
+	Tcl_CreateCommand(interp, "puts", WinPutsCmd, (ClientData)session_pointer, NULL);
+    Tcl_CreateCommand(interp, "getusername", WinGetUserName, (ClientData)session_pointer, NULL);
+#endif /* WIN32 */
 	Tk_DefineBitmap(interp, Tk_GetUid("ucl"), ucl_bits, ucl_width, ucl_height);
 	Tk_DefineBitmap(interp, Tk_GetUid("mic"), mic_bits, mic_width, mic_height);
 	Tk_DefineBitmap(interp, Tk_GetUid("mic_mute"), mic_mute_bits, mic_mute_width, mic_mute_height);
@@ -1148,7 +1145,7 @@ ui_init(session_struct *session_pointer, int argc, char **argv)
 	eval_check(interp, comm);
 	eval_check(interp, init_ui_script);
 
-#ifndef NDEBUG
+#if !defined(NDEBUG) && !defined(WIN32)
 	sprintf(comm, ".r.b.ucl configure -background salmon");
 	eval_check(interp, comm);
 	sprintf(comm, ".r.b.v   configure -background salmon");
